"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports._detectStackingTag = exports._detectSimpleTag = exports.FaTagRegEx = void 0;
// #region "Regular Expressions for Fontawesome-Tag"
const hyphenConnectedAlpha = "[a-z0-2]+(-[a-z0-2]+)*";
const faClassTag = `fa(-${hyphenConnectedAlpha})*`;
const faClassesTag = `${faClassTag}( +${faClassTag})*`;
const styleClassesTag = `\\.${hyphenConnectedAlpha}( +\\.${hyphenConnectedAlpha})*`;
const simpleFaTag = `:${faClassesTag}:`;
const styledFaTag = `\\[ *${simpleFaTag} *\\]\\{ *${styleClassesTag} *\\}`;
const faTag = `((${simpleFaTag})|(${styledFaTag}))`;
const simpleStackingFaTag = `\\[( *${simpleFaTag}){2,} *\\]`;
const stackingFaTag = `\\[( *${faTag}){2,} *\\]`;
// #endregion
/**
 * Regular Expression for Fontawesome-Tag
 */
exports.FaTagRegEx = {
    hyphenConnectedAlpha,
    simpleFaTag,
    styledFaTag,
    faTag,
    simpleStackingFaTag,
    stackingFaTag
};
//#region "private methods."
function _detectSimpleTag(source, ignoreStyled) {
    const result = _detectRegEx(source, faTag);
    if (result !== null) {
        // this is FaTag
        if (result.indexOf('{') > 0) {
            if (!ignoreStyled) {
                return {
                    kind: "simple", tag: result,
                    parsed: _parseFaTag(result)
                };
            }
        }
        else {
            return {
                kind: "simple", tag: result,
                parsed: _parseFaTag(result)
            };
        }
    }
    return null;
}
exports._detectSimpleTag = _detectSimpleTag;
function _detectStackingTag(source, ignoreStyled) {
    const result = _detectRegEx(source, stackingFaTag);
    if (result !== null) {
        // this is FaTag
        if (result.indexOf('{') > 0) {
            if (!ignoreStyled) {
                return {
                    kind: "stacking", tag: result,
                    parsed: _parseStackingTag(result)
                };
            }
        }
        else {
            return {
                kind: "stacking", tag: result,
                parsed: _parseStackingTag(result)
            };
        }
    }
    return null;
}
exports._detectStackingTag = _detectStackingTag;
function _parseFaTag(tag) {
    // read fa classes
    const faFound = (new RegExp(faClassesTag, "g")).exec(tag);
    // read style classes
    const styleFound = (new RegExp(styleClassesTag, "g")).exec(tag);
    return {
        faClasses: faFound[0],
        styleClasses: styleFound !== null ?
            styleFound[0] : null
    };
}
function _parseStackingTag(tag) {
    const found = [];
    const regex = new RegExp(faTag, "g");
    var fatag;
    while ((fatag = regex.exec(tag)) !== null) {
        found.push(_parseFaTag(fatag[0]));
    }
    return found;
}
function _detectRegEx(source, pattern) {
    const detected = (new RegExp(pattern, "g")).exec(source);
    return detected?.index === 0 ? detected[0] : null;
}
//#endregion
