"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StackingTokenizer = exports.FaTagTokenizer = exports.createTokenizer = void 0;
const TagDetector_1 = require("./TagDetector");
function createTokenizer(state, silent, option) {
    const pattern = (0, TagDetector_1.detectFaTagPattern)(state.src, state.pos, option.ignoreStyled ?? false);
    var tokenizer = null;
    if (pattern != null) {
        switch (pattern.kind) {
            case "simple":
            case "styled":
                tokenizer = new FaTagTokenizer(state, silent, pattern);
                break;
            case "simpleStacking":
            case "stacking":
                tokenizer = new StackingTokenizer(state, silent, pattern);
                break;
        }
    }
    return tokenizer;
}
exports.createTokenizer = createTokenizer;
class FaTokenizerBase {
    constructor(stateInline, silent, detectedTag) {
        this._stateInline = stateInline;
        this._silent = silent;
        this._detectedTag = detectedTag;
    }
    _pushStackingTag(stackingTag) {
        const stackingIconTag = this._stateInline.push('fa_icon_stacking_open', 'span', 1);
        stackingIconTag.attrPush(['class', "fa-stack"]);
        for (const fatag of stackingTag) {
            this._pushFaTag(fatag);
        }
        this._stateInline.push('fa_icon_stacking_close', 'span', -1);
    }
    _pushFaTag(fatag) {
        if (fatag.styleClasses != null) {
            const iconTag = this._stateInline.push('fa_icon_style_open', 'span', 1);
            iconTag.attrPush(['class', fatag.styleClasses.replaceAll('.', '')]);
        }
        const iconTag = this._stateInline.push('fa_icon_open', 'i', 1);
        iconTag.attrPush(['class', fatag.faClasses]);
        this._stateInline.push('fa_icon_close', 'i', -1);
        if (fatag.styleClasses != null) {
            this._stateInline.push('fa_icon_style_close', 'span', -1);
        }
    }
    run() {
        this._stateInline.pos = Math.min(this._stateInline.posMax, this._stateInline.pos + this._detectedTag.tag.length);
        if (!this._silent) {
            this._tokenize();
        }
    }
}
class FaTagTokenizer extends FaTokenizerBase {
    _tokenize() {
        this._pushFaTag(this._detectedTag.parsed);
    }
}
exports.FaTagTokenizer = FaTagTokenizer;
class StackingTokenizer extends FaTokenizerBase {
    _tokenize() {
        this._pushStackingTag(this._detectedTag.parsed);
    }
}
exports.StackingTokenizer = StackingTokenizer;
