"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addStyleClass = exports.tagToString = exports.detectFaTagPattern = exports.FaTagRegEx = void 0;
// #region "Regular Expressions for Fontawesome-Tag"
const hyphenConnectedAlpha = "[a-z0-2]+(-[a-z0-2]+)*";
const faClassTag = `fa(-${hyphenConnectedAlpha})*`;
const faClassesTag = `${faClassTag}( +${faClassTag})*`;
const styleClassesTag = `\\.${hyphenConnectedAlpha}( +\\.${hyphenConnectedAlpha})*`;
const simpleFaTag = `:${faClassesTag}:`;
const styledFaTag = `\\[ *${simpleFaTag} *\\]\\{ *${styleClassesTag} *\\}`;
const faTag = `((${simpleFaTag})|(${styledFaTag}))`;
const simpleStackingFaTag = `\\[( *${simpleFaTag}){2,} *\\]`;
const stackingFaTag = `\\[( *${faTag}){2,} *\\]`;
// #endregion
/**
 * Regular Expression for Fontawesome-Tag
 */
exports.FaTagRegEx = {
    hyphenConnectedAlpha,
    simpleFaTag,
    styledFaTag,
    faTag,
    simpleStackingFaTag,
    stackingFaTag
};
function detectFaTagPattern(str, pos, ignoreStyled) {
    // if not start with 0x3A(:) or 0x5B([) then returns null.
    if (str.charCodeAt(pos) !== 0x3A && str.charCodeAt(pos) !== 0x5B) {
        return null;
    }
    // slice from pos
    const source = str.slice(pos);
    // detect!
    let detected = _startsWithTag(source, ignoreStyled);
    if (detected === null) {
        detected = _startsWithStacking(source, ignoreStyled);
    }
    return detected;
}
exports.detectFaTagPattern = detectFaTagPattern;
function tagToString(tag) {
    if (tag.styleClasses !== null) {
        return `[:${tag.faClasses}:]{${tag.styleClasses}}`;
    }
    else {
        return `:${tag.faClasses}:`;
    }
}
exports.tagToString = tagToString;
function addStyleClass(tag, styleClass) {
    if (tag.styleClasses === null) {
        tag.styleClasses = "";
    }
    const classes = tag.styleClasses.split(" ")
        .filter(item => item !== '');
    classes.push(`.${styleClass}`);
    tag.styleClasses = classes.join(" ");
    return tag;
}
exports.addStyleClass = addStyleClass;
//#region "private methods."
function _startsWithTag(source, ignoreStyled) {
    const result = _startsWithRegEx(source, faTag);
    if (result !== null) {
        // this is FaTag
        if (result.indexOf('{') > 0) {
            if (!ignoreStyled) {
                return {
                    kind: "styled", tag: result,
                    parsed: _parseFaTag(result)
                };
            }
        }
        else {
            return {
                kind: "simple", tag: result,
                parsed: _parseFaTag(result)
            };
        }
    }
    return null;
}
function _startsWithStacking(source, ignoreStyled) {
    const result = _startsWithRegEx(source, stackingFaTag);
    if (result !== null) {
        // this is FaTag
        if (result.indexOf('{') > 0) {
            if (!ignoreStyled) {
                return {
                    kind: "stacking", tag: result,
                    parsed: _parseStackingTag(result)
                };
            }
        }
        else {
            return {
                kind: "simpleStacking", tag: result,
                parsed: _parseStackingTag(result)
            };
        }
    }
    return null;
}
function _parseFaTag(tag) {
    // read fa classes
    const faFound = (new RegExp(faClassesTag, "g")).exec(tag);
    if (faFound !== null) {
        // read style classes
        const styleFound = (new RegExp(styleClassesTag, "g")).exec(tag);
        if (styleFound !== null) {
            return { faClasses: faFound[0], styleClasses: styleFound[0] };
        }
        return { faClasses: faFound[0], styleClasses: null };
    }
    else {
        throw new Error('Unexpected!!, fa classes not found.');
    }
}
function _parseStackingTag(tag) {
    const found = [];
    const regex = new RegExp(faTag, "g");
    var fatag;
    while ((fatag = regex.exec(tag)) !== null) {
        found.push(_parseFaTag(fatag[0]));
    }
    return found;
}
function _startsWithRegEx(source, pattern) {
    const detected = (new RegExp(pattern, "g")).exec(source);
    return detected?.index === 0 ? detected[0] : null;
}
