"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerFontawesomeTag = exports.fontawesome = exports.FontawesomeTagRule = void 0;
/**
 * engine class
 */
class FontawesomeTagRule {
    // #endregion
    // #region "constructor"
    constructor(state, silent) {
        this._hasFontawesomeTag = null;
        this._startPosOfFaClasses = -1;
        this._endPosOfFaTag = -1;
        this._state = state;
        this._silent = silent;
    }
    // #endregion
    // #region "public propertis"
    get hasFontawesomeTag() {
        if (this._hasFontawesomeTag === null) {
            this._hasFontawesomeTag = this._detectTag();
        }
        return this._hasFontawesomeTag;
    }
    get startPosOfFaClasses() {
        if (!this.hasFontawesomeTag) {
            return -1;
        }
        return this._startPosOfFaClasses;
    }
    get endPosOfFaClasses() {
        if (!this.hasFontawesomeTag) {
            return -1;
        }
        return this.endPosOfFaTag - 1;
    }
    get endPosOfFaTag() {
        if (!this.hasFontawesomeTag) {
            return -1;
        }
        return this._endPosOfFaTag;
    }
    get faClasses() {
        if (!this.hasFontawesomeTag) {
            return null;
        }
        return this._state.src.slice(this.startPosOfFaClasses, this.endPosOfFaClasses + 1);
    }
    // #endregion
    // #region "public method"
    run() {
        if (!this.hasFontawesomeTag) {
            return false;
        }
        if (!this._silent) {
            const iconTag = this._state.push('fa_icon_open', 'i', 1);
            if (this.faClasses != null) {
                iconTag.attrPush(['class', this.faClasses]);
            }
            this._state.push('fa_icon_close', 'i', -1);
        }
        this._state.pos = this.endPosOfFaTag + 1;
        return true;
    }
    // #endregion
    // #region "private properties"
    get _tokenMatchLong() {
        return this._state.posMax - this._state.pos > 3;
    }
    get _startsWithMark() {
        // ':'
        var found = (this._state.src.charCodeAt(this._state.pos) === 0x3A
            // 'f'
            && this._state.src.charCodeAt(this._state.pos + 1) === 0x66
            // 'a'
            && this._state.src.charCodeAt(this._state.pos + 2) === 0x61
            // ' ' or '-'
            && (this._state.src.charCodeAt(this._state.pos + 3) === 0x20
                || this._state.src.charCodeAt(this._state.pos + 3) === 0x2D));
        if (found) {
            this._startPosOfFaClasses = this._state.pos + 1;
        }
        return found;
    }
    get _canFindEndOfMark() {
        var found = false;
        for (var index = this._startPosOfFaClasses; index <= this._state.posMax; index++) {
            if (this._state.src.charCodeAt(index) === 0x3A) {
                // find ':'
                found = true;
                this._endPosOfFaTag = index;
                break;
            }
        }
        return found;
    }
    // #endregion
    // #region "private method"
    _detectTag() {
        if (!(this._tokenMatchLong && this._startsWithMark && this._canFindEndOfMark)) {
            return false;
        }
        return true;
    }
}
exports.FontawesomeTagRule = FontawesomeTagRule;
/**
 * Entry point of this plugin-engine.
 * Decide the tag is included in current callet of markdown syntax,
 * And inprove callet to next token.
 * @param state - inline parser state
 * @param silent - parse mode (is silent or not)
 * @returns fontawesome-tag is included in current callet wether or not.
 */
function fontawesome(state, silent) {
    return (new FontawesomeTagRule(state, silent)).run();
}
exports.fontawesome = fontawesome;
function registerFontawesomeTag(md, opt) {
    md.inline.ruler.push('fontawesome', fontawesome);
}
exports.registerFontawesomeTag = registerFontawesomeTag;
